/*
 *	Copyright (C) 2007, 2008. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file qvip.h
/// @brief File from the QVision library.
/// @author PARP Research Group. University of Murcia, Spain.

#ifndef QVIP_H
#define QVIP_H

#include <iostream>

#include <qvdefines.h>
#include <QVImage>
//#include <qvcore/qvimage.h>

/*!
@brief Image Processing.

See \ref Structure for a detailed explanation of the structure of this package.

*/

/// @ingroup qvip
/// @todo document this
void FilterHarrisCornerResponseImage(const QVImage<uChar> &image, QVImage<sFloat> &result, const QPoint &destROIOffset = QPoint(0,0));

/// @ingroup qvip
/// @todo document this
void FilterDoG(const QVImage<uChar> &image, QVImage<sFloat> &result);

/// @ingroup qvip
/// @deprecated see @ref FilterHessianCornerResponseImage
/// @todo document this
void SobelCornerResponseImage(const QVImage<sFloat> &image, QVImage<sFloat> &result);

/// @ingroup qvip
/// @todo document this
void FilterHessianCornerResponseImage(const QVImage<sFloat> &image, QVImage<sFloat> &result, const QPoint &destROIOffset = QPoint(0,0));

/// @ingroup qvip
/// @todo document this
int myFloodFill(QVImage<uChar> &image, uInt x, uInt y, uInt value, uInt minVal, uInt maxVal);

/// @ingroup qvip
/// @todo document this
void FilterEqualizeHistogram(const QVImage<uChar,1> &image, QVImage<uChar,1> &equalized, const QPoint &destROIOffset = QPoint(0,0));

/// @ingroup qvip
/// @todo document this
void FilterEqualizeHistogram(const QVImage<sFloat,1> &image, QVImage<sFloat,1> &equalized, const QPoint &destROIOffset = QPoint(0,0));

/// @ingroup qvip
class QVComponentTree;
void FilterPruneComponentTreeSmallRegions(QVImage<uChar> &image, QVComponentTree &componentTree, uInt area);

#include <qvmath/qvvector.h>
#include <QPoint>
/// @ingroup qvip
void FilterSeparable(const QVImage<sFloat, 1> &image, QVImage<sFloat, 1> &dest,
	const QVVector &rowFilter, const QVVector &colFilter, const QPoint &destROIOffset = QPoint(0,0));

/// @brief Generates a binary image of local grey-scale maximal value pixels.
/// @ingroup qvip
///
/// This function receives a QVImage, and generates a binary image where each pixel is set to IPP_MAX_8U if the pixel in the
/// original image is strict maximal in value regarding to pixels in a vicinity window of colMaskSize width, and rowMaskSize
/// height.
///
/// \todo
///	- Fix resulting image ROI, obtain maximums restricted to the ROI.
///	- Use IPP functions to obtain the result.
///
/// @param src source image.
/// @param dest binary image that will contain maximal values.
/// @param colMaskSize width of the vicinity.
/// @param rowMaskSize height of the vicinity.
void FilterLocalMax(const QVImage<sFloat> &src, QVImage<uChar> &dest, uInt colMaskSize, uInt rowMaskSize, sFloat threshold = 0);

#endif

