/*
 *	Copyright (C) 2007, 2008. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file qvippworkers.cpp
/// @brief File from the QVision library.
/// @author PARP Research Group. University of Murcia, Spain.

#include <qvippworkers.h>
#include <qvip/qvipp/macros.h>

////////////////////////////////////
// Macros
// IppiMaskSize
#define ADD_IPPI_MASK_SIZE_PROPERTY(Name)		\
	QVIndexedStringList Name ## _choice;		\
	/*Name ## _choice.append("ippMskSize1x3");	\
	Name ## _choice.append("ippMskSize1x5");	\
	Name ## _choice.append("ippMskSize3x1");*/	\
	Name ## _choice.append("ippMskSize3x3");	\
	/*Name ## _choice.append("ippMskSize5x1");*/	\
	Name ## _choice.append("ippMskSize5x5");	\
	addProperty< QVIndexedStringList >("" # Name, inputFlag, Name ## _choice);

#define	GET_IPPI_MASK_SIZE_PROPERTY(Name)								\
	const QString Name ## _indexedStringList =							\
				getPropertyValue< QVIndexedStringList >("" #Name).getCurrent();		\
	const IppiMaskSize Name =									\
				(Name ## _indexedStringList == "ippMskSize1x3")? ippMskSize1x3:		\
				(Name ## _indexedStringList == "ippMskSize1x5")? ippMskSize1x5:		\
				(Name ## _indexedStringList == "ippMskSize3x1")? ippMskSize3x1:		\
				(Name ## _indexedStringList == "ippMskSize3x3")? ippMskSize3x3:		\
				(Name ## _indexedStringList == "ippMskSize5x1")? ippMskSize5x1:		\
				ippMskSize5x5;

// IppCmpOp
#define ADD_IPP_CMP_OP_PROPERTY(Name)			\
	QVIndexedStringList Name ## _choice;		\
	Name ## _choice.append("ippCmpLessEq");		\
	Name ## _choice.append("ippCmpEq");		\
	Name ## _choice.append("ippCmpGreaterEq");	\
	Name ## _choice.append("ippCmpGreater");	\
	addProperty< QVIndexedStringList >("" # Name, inputFlag, Name ## _choice);

#define	GET_IPP_CMP_OP_PROPERTY(Name)									\
	const QString Name ## _indexedStringList =							\
				getPropertyValue< QVIndexedStringList >("" #Name).getCurrent();		\
	const IppCmpOp Name =									\
				(Name ## _indexedStringList == "ippCmpLessEq")? ippCmpLessEq:		\
				(Name ## _indexedStringList == "ippCmpEq")? ippCmpEq:			\
				(Name ## _indexedStringList == "ippCmpGreaterEq")? ippCmpGreaterEq:	\
				ippCmpGreater;

#define	CREATE_OUTPUT_IMAGE(Type, Channels, Dest, Src, DestROIOffset)	\
	QVImage<Type, Channels> Dest(Src.getROI().width() + DestROIOffset.x(), Src.getROI().height() + DestROIOffset.y());

////////////////////////////////////
// qvippCopyP3C3Worker<uChar,1>
template <> qvippCopyP3C3Worker<uChar>::qvippCopyP3C3Worker(QString name): QVWorker(name)
	{
	addProperty< QVImage<uChar,1> >("src1", inputFlag|outputFlag);
	addProperty< QVImage<uChar,1> >("src2", inputFlag|outputFlag);
	addProperty< QVImage<uChar,1> >("src3", inputFlag|outputFlag);
	addProperty< QVImage<uChar,3> >("dest", outputFlag);

	addProperty< QPoint >("destROIOffset", inputFlag, QPoint(0,0));
	}

template <> void qvippCopyP3C3Worker<uChar>::iterate()
	{
	// Read input properties
	const QVImage<uChar,1>	src1 = getPropertyValue< QVImage<uChar,1> >("src1"),
				src2 = getPropertyValue< QVImage<uChar,1> >("src2"),
				src3 = getPropertyValue< QVImage<uChar,1> >("src3");
	const QPoint destROIOffset = getPropertyValue< QPoint >("destROIOffset");
	timeFlag("Read input parameters");

	//CHECK_SAME_ROISIZE(src1, src2);
	//CHECK_SAME_ROISIZE(src2, src3);

	// Apply dilate 3x3 filter	
	CREATE_OUTPUT_IMAGE(uChar, 3, dest, src1, destROIOffset);
	Copy(src1, src2, src3, dest, destROIOffset);

	setPropertyValue< QVImage<uChar,3> >("dest", dest);
	timeFlag("Write output parameters");
	}


// qvippCopyC3P3Worker<uChar,1>
template <> qvippCopyC3P3Worker<uChar>::qvippCopyC3P3Worker(QString name): QVWorker(name)
	{
	addProperty< QVImage<uChar,3> >("src", inputFlag|outputFlag);

	addProperty< QVImage<uChar,1> >("dest1", outputFlag);
	addProperty< QVImage<uChar,1> >("dest2", outputFlag);
	addProperty< QVImage<uChar,1> >("dest3", outputFlag);
	addProperty< QPoint >("destROIOffset", inputFlag, QPoint(0,0));
	}

template <> void qvippCopyC3P3Worker<uChar>::iterate()
	{
	// Read input properties
	const QVImage<uChar,3>	src = getPropertyValue< QVImage<uChar,3> >("src");
	const QPoint destROIOffset = getPropertyValue< QPoint >("destROIOffset");
	timeFlag("Read input parameters");

	// Apply dilate 3x3 filter	
	CREATE_OUTPUT_IMAGE(uChar, 1, dest1, src, destROIOffset);
	CREATE_OUTPUT_IMAGE(uChar, 1, dest2, src, destROIOffset);
	CREATE_OUTPUT_IMAGE(uChar, 1, dest3, src, destROIOffset);
	Copy(src, dest1, dest2, dest3, destROIOffset);

	setPropertyValue< QVImage<uChar,1> >("dest1", dest1);
	setPropertyValue< QVImage<uChar,1> >("dest2", dest2);
	setPropertyValue< QVImage<uChar,1> >("dest3", dest3);
	timeFlag("Write output parameters");
	}

// qvippDilate3x3Worker<uChar,1>
template <> qvippDilate3x3Worker<uChar,1>::qvippDilate3x3Worker(QString name): QVWorker(name)
	{
	addProperty< QVImage<uChar,1> >("src", inputFlag|outputFlag);
	addProperty< QVImage<uChar,1> >("dest", outputFlag);

	addProperty< QPoint >("destROIOffset", inputFlag, QPoint(0,0));
	}

template <> void qvippDilate3x3Worker<uChar,1>::iterate()
	{
	// Read input properties
	const QVImage<uChar,1> src = getPropertyValue< QVImage<uChar,1> >("src");
	const QPoint destROIOffset = getPropertyValue< QPoint >("destROIOffset");
	timeFlag("Read input parameters");

	// Apply dilate 3x3 filter	
	CREATE_OUTPUT_IMAGE(uChar, 1, dest, src, destROIOffset);
	//QVImage<uChar,1> dest(src.getCols() + destROIOffset.x(), src.getRows() + destROIOffset.y());
	Dilate3x3(src, dest);

	setPropertyValue< QVImage<uChar,1> >("dest", dest);
	timeFlag("Write output parameters");
	}

// qvippErode3x3Worker<uChar,1>
template <> qvippErode3x3Worker<uChar,1>::qvippErode3x3Worker(QString name): QVWorker(name)
	{
	addProperty< QVImage<uChar,1> >("src", inputFlag|outputFlag);
	addProperty< QVImage<uChar,1> >("dest", outputFlag);

	addProperty< QPoint >("destROIOffset", inputFlag, QPoint(0,0));
	}

template <> void qvippErode3x3Worker<uChar,1>::iterate()
	{
	// Read input properties
	const QVImage<uChar,1> src = getPropertyValue< QVImage<uChar,1> >("src");
	const QPoint destROIOffset = getPropertyValue< QPoint >("destROIOffset");
	timeFlag("Read input parameters");

	// Apply dilate 3x3 filter	
	CREATE_OUTPUT_IMAGE(uChar, 1, dest, src, destROIOffset);
	Erode3x3(src, dest);

	setPropertyValue< QVImage<uChar,1> >("dest", dest);
	timeFlag("Write output parameters");
	}


// qvippComputeThreshold_OtsuWorker<uChar,1>
template <> qvippComputeThreshold_OtsuWorker<uChar,1>::qvippComputeThreshold_OtsuWorker(QString name): QVWorker(name)
	{
	addProperty< QVImage<uChar,1> >("src", inputFlag|outputFlag);

	addProperty< int >("Threshold Otsu", outputFlag);
	}

template <> void qvippComputeThreshold_OtsuWorker<uChar,1>::iterate()
	{
	// Read input properties
	const QVImage<uChar,1> src = getPropertyValue< QVImage<uChar,1> >("src");
	timeFlag("Read input parameters");

	// Apply random filter	
	uChar threshold;
	ComputeThreshold_Otsu(src, threshold);

	setPropertyValue< int >("Threshold Otsu", threshold);
	timeFlag("Write output parameters");
	}


// qvippAddRandUniform_DirectWorker<uChar,1>
template <> qvippAddRandUniform_DirectWorker<uChar,1>::qvippAddRandUniform_DirectWorker(QString name): QVWorker(name), seed(0)
	{
	addProperty< QVImage<uChar,1> >("src", inputFlag|outputFlag);
	addProperty< QVImage<uChar,1> >("dest", outputFlag);
	addProperty< int >("Low", inputFlag, 10, "", 0, 255);
	addProperty< int >("High", inputFlag, 30, "", 0, 255);

	addProperty< QPoint >("destROIOffset", inputFlag, QPoint(0,0));
	}

template <> void qvippAddRandUniform_DirectWorker<uChar,1>::iterate()
	{
	// Read input properties
	QVImage<uChar,1> src = getPropertyValue< QVImage<uChar,1> >("src");
	const uChar	low = getPropertyValue< uChar >("Low"),
			high = getPropertyValue< uChar >("High");
	const QPoint destROIOffset = getPropertyValue< QPoint >("destROIOffset");
	timeFlag("Read input parameters");

	// Apply random filter	
	AddRandUniform_Direct(src, low, high, seed);

	seed++;

	setPropertyValue< QVImage<uChar,1> >("dest", src);
	timeFlag("Write output parameters");
	}

// qvippFilterBoxWorker<uChar,1>
template <> qvippFilterBoxWorker<uChar,1>::qvippFilterBoxWorker(QString name): QVWorker(name)
	{
	addProperty< QVImage<uChar,1> >("src", inputFlag|outputFlag);
	addProperty< QVImage<uChar,1> >("dest", outputFlag);
	addProperty< int >("Mask Width", inputFlag, 1, "", 1, 64);
	addProperty< int >("Mask Height", inputFlag, 1, "", 1, 64);

	addProperty< QPoint >("destROIOffset", inputFlag, QPoint(0,0));
	}

template <> void qvippFilterBoxWorker<uChar,1>::iterate()
	{
	// Read input properties
	const QVImage<uChar,1> src = getPropertyValue< QVImage<uChar,1> >("src");
	const int	maskWidth = getPropertyValue< int >("Mask Width"),
			maskHeight = getPropertyValue< int >("Mask Height");
	const QPoint destROIOffset = getPropertyValue< QPoint >("destROIOffset");
	timeFlag("Read input parameters");

	// Create output variables
	CREATE_OUTPUT_IMAGE(uChar, 1, dest, src, destROIOffset);

	FilterBox(src, dest, QSize(maskWidth, maskHeight), QPoint(0,0), destROIOffset);
	timeFlag("Call to ippiFilterBox_8u_C1R");

	setPropertyValue< QVImage<uChar,1> >("dest", dest);
	timeFlag("Write output parameters");
	}

// qvippFilterBoxWorker<uChar,3>
template <> qvippFilterBoxWorker<uChar,3>::qvippFilterBoxWorker(QString name): QVWorker(name)
	{
	addProperty< QVImage<uChar,3> >("src", inputFlag|outputFlag);
	addProperty< QVImage<uChar,3> >("dest", outputFlag);
	addProperty< int >("Mask Width", inputFlag, 1, "", 1, 64);
	addProperty< int >("Mask Height", inputFlag, 1, "", 1, 64);

	addProperty< QPoint >("destROIOffset", inputFlag, QPoint(0,0));
	}

template <> void qvippFilterBoxWorker<uChar,3>::iterate()
	{
	// Read input properties
	const QVImage<uChar,3> src = getPropertyValue< QVImage<uChar,3> >("src");
	const int	maskWidth = getPropertyValue< int >("Mask Width"),
			maskHeight = getPropertyValue< int >("Mask Height");
	const QPoint destROIOffset = getPropertyValue< QPoint >("destROIOffset");
	timeFlag("Read input parameters");

	// Create output variables
	QVImage<uChar,3> dest(src.getCols() + destROIOffset.x(), src.getRows() + destROIOffset.y());

	FilterBox(src, dest, QSize(maskWidth, maskHeight), QPoint(0,0), destROIOffset);
	timeFlag("Call to ippiFilterBox_8u_C1R");

	setPropertyValue< QVImage<uChar,3> >("dest", dest);
	timeFlag("Write output parameters");
	}


// qvippFilterMedianWorker<uChar,1>
template <> qvippFilterMedianWorker<uChar,1>::qvippFilterMedianWorker(QString name): QVWorker(name)
	{
	addProperty< QVImage<uChar,1> >("src", inputFlag|outputFlag);
	addProperty< QVImage<uChar,1> >("dest", outputFlag);
	addProperty< int >("Mask Width", inputFlag, 1, "", 1, 64);
	addProperty< int >("Mask Height", inputFlag, 1, "", 1, 64);

	addProperty< QPoint >("destROIOffset", inputFlag, QPoint(0,0));
	}

template <> void qvippFilterMedianWorker<uChar,1>::iterate()
	{
	// Read input properties
	const QVImage<uChar,1> src = getPropertyValue< QVImage<uChar,1> >("src");
	const int	maskWidth = getPropertyValue< int >("Mask Width"),
			maskHeight = getPropertyValue< int >("Mask Height");
	const QPoint destROIOffset = getPropertyValue< QPoint >("destROIOffset");
	timeFlag("Read input parameters");

	// Create output variables
	CREATE_OUTPUT_IMAGE(uChar, 1, dest, src, destROIOffset);

	FilterMedian(src, dest, QSize(maskWidth, maskHeight), QPoint(0,0), destROIOffset);
	timeFlag("Call to ippiFilterMedian_8u_C1R");

	setPropertyValue< QVImage<uChar,1> >("dest", dest);
	timeFlag("Write output parameters");
	}

// qvippFilterMedianWorker<uChar,3>
template <> qvippFilterMedianWorker<uChar,3>::qvippFilterMedianWorker(QString name): QVWorker(name)
	{
	addProperty< QVImage<uChar,3> >("src", inputFlag|outputFlag);
	addProperty< QVImage<uChar,3> >("dest", outputFlag);
	addProperty< int >("Mask Width", inputFlag, 1, "", 1, 64);
	addProperty< int >("Mask Height", inputFlag, 1, "", 1, 64);

	addProperty< QPoint >("destROIOffset", inputFlag, QPoint(0,0));
	}

template <> void qvippFilterMedianWorker<uChar,3>::iterate()
	{
	// Read input properties
	const QVImage<uChar,3> src = getPropertyValue< QVImage<uChar,3> >("src");
	const int	maskWidth = getPropertyValue< int >("Mask Width"),
			maskHeight = getPropertyValue< int >("Mask Height");
	const QPoint destROIOffset = getPropertyValue< QPoint >("destROIOffset");
	timeFlag("Read input parameters");

	// Create output variables
	QVImage<uChar,3> dest(src.getCols() + destROIOffset.x(), src.getRows() + destROIOffset.y());

	FilterMedian(src, dest, QSize(maskWidth, maskHeight), QPoint(0,0), destROIOffset);
	timeFlag("Call to ippiFilterMedian_8u_C1R");

	setPropertyValue< QVImage<uChar,3> >("dest", dest);
	timeFlag("Write output parameters");
	}

// qvippFilterMaxWorker<uChar,1>
template <> qvippFilterMaxWorker<uChar,1>::qvippFilterMaxWorker(QString name): QVWorker(name)
	{
	addProperty< QVImage<uChar,1> >("src", inputFlag|outputFlag);
	addProperty< QVImage<uChar,1> >("dest", outputFlag);
	addProperty< int >("Mask Width", inputFlag, 1, "", 1, 64);
	addProperty< int >("Mask Height", inputFlag, 1, "", 1, 64);

	addProperty< QPoint >("destROIOffset", inputFlag, QPoint(0,0));
	}

template <> void qvippFilterMaxWorker<uChar,1>::iterate()
	{
	// Read input properties
	const QVImage<uChar,1> src = getPropertyValue< QVImage<uChar,1> >("src");
	const int	maskWidth = getPropertyValue< int >("Mask Width"),
			maskHeight = getPropertyValue< int >("Mask Height");
	const QPoint destROIOffset = getPropertyValue< QPoint >("destROIOffset");
	timeFlag("Read input parameters");

	// Create output variables
	CREATE_OUTPUT_IMAGE(uChar, 1, dest, src, destROIOffset);

	FilterMax(src, dest, QSize(maskWidth, maskHeight), QPoint(0,0), destROIOffset);
	timeFlag("Call to ippiFilterMax_8u_C1R");

	setPropertyValue< QVImage<uChar,1> >("dest", dest);
	timeFlag("Write output parameters");
	}

// qvippFilterMaxWorker<uChar,3>
template <> qvippFilterMaxWorker<uChar,3>::qvippFilterMaxWorker(QString name): QVWorker(name)
	{
	addProperty< QVImage<uChar,3> >("src", inputFlag|outputFlag);
	addProperty< QVImage<uChar,3> >("dest", outputFlag);
	addProperty< int >("Mask Width", inputFlag, 1, "", 1, 64);
	addProperty< int >("Mask Height", inputFlag, 1, "", 1, 64);

	addProperty< QPoint >("destROIOffset", inputFlag, QPoint(0,0));
	}

template <> void qvippFilterMaxWorker<uChar,3>::iterate()
	{
	// Read input properties
	const QVImage<uChar,3> src = getPropertyValue< QVImage<uChar,3> >("src");
	const int	maskWidth = getPropertyValue< int >("Mask Width"),
			maskHeight = getPropertyValue< int >("Mask Height");
	const QPoint destROIOffset = getPropertyValue< QPoint >("destROIOffset");
	timeFlag("Read input parameters");

	// Create output variables
	QVImage<uChar,3> dest(src.getCols() + destROIOffset.x(), src.getRows() + destROIOffset.y());

	FilterMax(src, dest, QSize(maskWidth, maskHeight), QPoint(0,0), destROIOffset);
	timeFlag("Call to ippiFilterMax_8u_C1R");

	setPropertyValue< QVImage<uChar,3> >("dest", dest);
	timeFlag("Write output parameters");
	}

// qvippFilterMinWorker<uChar,1>
template <> qvippFilterMinWorker<uChar,1>::qvippFilterMinWorker(QString name): QVWorker(name)
	{
	addProperty< QVImage<uChar,1> >("src", inputFlag|outputFlag);
	addProperty< QVImage<uChar,1> >("dest", outputFlag);
	addProperty< int >("Mask Width", inputFlag, 1, "", 1, 64);
	addProperty< int >("Mask Height", inputFlag, 1, "", 1, 64);

	addProperty< QPoint >("destROIOffset", inputFlag, QPoint(0,0));
	}

template <> void qvippFilterMinWorker<uChar,1>::iterate()
	{
	// Read input properties
	const QVImage<uChar,1> src = getPropertyValue< QVImage<uChar,1> >("src");
	const int	maskWidth = getPropertyValue< int >("Mask Width"),
			maskHeight = getPropertyValue< int >("Mask Height");
	const QPoint destROIOffset = getPropertyValue< QPoint >("destROIOffset");
	timeFlag("Read input parameters");

	// Create output variables
	CREATE_OUTPUT_IMAGE(uChar, 1, dest, src, destROIOffset);

	FilterMin(src, dest, QSize(maskWidth, maskHeight), QPoint(0,0), destROIOffset);
	timeFlag("Call to ippiFilterMin_8u_C1R");

	setPropertyValue< QVImage<uChar,1> >("dest", dest);
	timeFlag("Write output parameters");
	}

// qvippFilterMinWorker<uChar,3>
template <> qvippFilterMinWorker<uChar,3>::qvippFilterMinWorker(QString name): QVWorker(name)
	{
	addProperty< QVImage<uChar,3> >("src", inputFlag|outputFlag);
	addProperty< QVImage<uChar,3> >("dest", outputFlag);
	addProperty< int >("Mask Width", inputFlag, 1, "", 1, 64);
	addProperty< int >("Mask Height", inputFlag, 1, "", 1, 64);

	addProperty< QPoint >("destROIOffset", inputFlag, QPoint(0,0));
	}

template <> void qvippFilterMinWorker<uChar,3>::iterate()
	{
	// Read input properties
	const QVImage<uChar,3> src = getPropertyValue< QVImage<uChar,3> >("src");
	const int	maskWidth = getPropertyValue< int >("Mask Width"),
			maskHeight = getPropertyValue< int >("Mask Height");
	const QPoint destROIOffset = getPropertyValue< QPoint >("destROIOffset");
	timeFlag("Read input parameters");

	// Create output variables
	QVImage<uChar,3> dest(src.getCols() + destROIOffset.x(), src.getRows() + destROIOffset.y());

	FilterMin(src, dest, QSize(maskWidth, maskHeight), QPoint(0,0), destROIOffset);
	timeFlag("Call to ippiFilterMin_8u_C1R");

	setPropertyValue< QVImage<uChar,3> >("dest", dest);
	timeFlag("Write output parameters");
	}

// qvippFilterGaussWorker<uChar,1>
template <> qvippFilterGaussWorker<uChar,1>::qvippFilterGaussWorker(QString name): QVWorker(name)
	{
	addProperty< QVImage<uChar,1> >("src", inputFlag|outputFlag);
	addProperty< QVImage<uChar,1> >("dest", outputFlag);
	ADD_IPPI_MASK_SIZE_PROPERTY(maskSize);
	addProperty< QPoint >("destROIOffset", inputFlag, QPoint(0,0));
	}

template <> void qvippFilterGaussWorker<uChar,1>::iterate()
	{
	// Read input properties
	const QVImage<uChar,1> src = getPropertyValue< QVImage<uChar,1> >("src");
	GET_IPPI_MASK_SIZE_PROPERTY(maskSize);
	const QPoint destROIOffset = getPropertyValue< QPoint >("destROIOffset");
	timeFlag("Read input parameters");

	// Create output variables
	CREATE_OUTPUT_IMAGE(uChar, 1, dest, src, destROIOffset);

	FilterGauss(src, dest, maskSize, destROIOffset);
	timeFlag("Call to ippiFilterGauss_8u_C1R");

	setPropertyValue< QVImage<uChar,1> >("dest", dest);
	timeFlag("Write output parameters");
	}

// qvippFilterHipassWorker<uChar,1>
template <> qvippFilterHipassWorker<uChar,1>::qvippFilterHipassWorker(QString name): QVWorker(name)
	{
	addProperty< QVImage<uChar,1> >("src", inputFlag|outputFlag);
	addProperty< QVImage<uChar,1> >("dest", outputFlag);
	ADD_IPPI_MASK_SIZE_PROPERTY(maskSize);
	addProperty< QPoint >("destROIOffset", inputFlag, QPoint(0,0));
	}

template <> void qvippFilterHipassWorker<uChar,1>::iterate()
	{
	// Read input properties
	const QVImage<uChar,1> src = getPropertyValue< QVImage<uChar,1> >("src");
	GET_IPPI_MASK_SIZE_PROPERTY(maskSize);
	const QPoint destROIOffset = getPropertyValue< QPoint >("destROIOffset");
	timeFlag("Read input parameters");

	// Create output variables
	CREATE_OUTPUT_IMAGE(uChar, 1, dest, src, destROIOffset);

	FilterHipass(src, dest, maskSize, destROIOffset);
	timeFlag("Call to ippiFilterHipass_8u_C1R");

	setPropertyValue< QVImage<uChar,1> >("dest", dest);
	timeFlag("Write output parameters");
	}

// qvippFilterGaussWorker<uChar,3>
template <> qvippFilterGaussWorker<uChar,3>::qvippFilterGaussWorker(QString name): QVWorker(name)
	{
	addProperty< QVImage<uChar,3> >("src", inputFlag|outputFlag);
	addProperty< QVImage<uChar,3> >("dest", outputFlag);
	ADD_IPPI_MASK_SIZE_PROPERTY(maskSize);
	addProperty< QPoint >("destROIOffset", inputFlag, QPoint(0,0));
	}

template <> void qvippFilterGaussWorker<uChar,3>::iterate()
	{
	// Read input properties
	const QVImage<uChar,3> src = getPropertyValue< QVImage<uChar,3> >("src");
	GET_IPPI_MASK_SIZE_PROPERTY(maskSize);
	const QPoint destROIOffset = getPropertyValue< QPoint >("destROIOffset");
	timeFlag("Read input parameters");

	// Create output variables
	QVImage<uChar,3> dest(src.getCols() + destROIOffset.x(), src.getRows() + destROIOffset.y());

	FilterGauss(src, dest, maskSize, destROIOffset);
	timeFlag("Call to ippiFilterGauss_8u_C1R");

	setPropertyValue< QVImage<uChar,3> >("dest", dest);
	timeFlag("Write output parameters");
	}

// qvippFilterHipassWorker<uChar,3>
template <> qvippFilterHipassWorker<uChar,3>::qvippFilterHipassWorker(QString name): QVWorker(name)
	{
	addProperty< QVImage<uChar,3> >("src", inputFlag|outputFlag);
	addProperty< QVImage<uChar,3> >("dest", outputFlag);
	ADD_IPPI_MASK_SIZE_PROPERTY(maskSize);
	addProperty< QPoint >("destROIOffset", inputFlag, QPoint(0,0));
	}

template <> void qvippFilterHipassWorker<uChar,3>::iterate()
	{
	// Read input properties
	const QVImage<uChar,3> src = getPropertyValue< QVImage<uChar,3> >("src");
	GET_IPPI_MASK_SIZE_PROPERTY(maskSize);
	const QPoint destROIOffset = getPropertyValue< QPoint >("destROIOffset");
	timeFlag("Read input parameters");

	// Create output variables
	QVImage<uChar,3> dest(src.getCols() + destROIOffset.x(), src.getRows() + destROIOffset.y());

	FilterHipass(src, dest, maskSize, destROIOffset);
	timeFlag("Call to ippiFilterHipass_8u_C1R");

	setPropertyValue< QVImage<uChar,3> >("dest", dest);
	timeFlag("Write output parameters");
	}

// qvippAddWorker<uChar,1>
template <> qvippAddWorker<uChar,1>::qvippAddWorker(QString name): QVWorker(name)
	{
	addProperty< QVImage<uChar,1> >("src1", inputFlag|outputFlag);
	addProperty< QVImage<uChar,1> >("src2", inputFlag|outputFlag);
	addProperty< int >("scale", inputFlag, 1, "", 0, 16);
	addProperty< QVImage<uChar,1> >("dest", outputFlag);
	addProperty< QPoint >("destROIOffset", inputFlag, QPoint(0,0));
	}

template <> void qvippAddWorker<uChar,1>::iterate()
	{
	// Read input properties
	const QVImage<uChar,1> src1 = getPropertyValue< QVImage<uChar,1> >("src1");
	const QVImage<uChar,1> src2 = getPropertyValue< QVImage<uChar,1> >("src2");
	const int scale = getPropertyValue< int >("scale");
	const QPoint destROIOffset = getPropertyValue< QPoint >("destROIOffset");
	timeFlag("Read input parameters");

	// Create output variables
	QVImage<uChar,1> dest(src1.getCols() + destROIOffset.x(), src1.getRows() + destROIOffset.y());

	Add(src1, src2, dest, scale, destROIOffset);
	timeFlag("Call to ippiAdd_8u_C1R");

	setPropertyValue< QVImage<uChar,1> >("dest", dest);
	timeFlag("Write output parameters");
	}

// qvippAddWorker<uChar,3>
template <> qvippAddWorker<uChar,3>::qvippAddWorker(QString name): QVWorker(name)
	{
	addProperty< QVImage<uChar,3> >("src1", inputFlag|outputFlag);
	addProperty< QVImage<uChar,3> >("src2", inputFlag|outputFlag);
	addProperty< int >("scale", inputFlag, 1, "", 0, 16);
	addProperty< QVImage<uChar,3> >("dest", outputFlag);
	addProperty< QPoint >("destROIOffset", inputFlag, QPoint(0,0));
	}

template <> void qvippAddWorker<uChar,3>::iterate()
	{
	// Read input properties
	const QVImage<uChar,3> src1 = getPropertyValue< QVImage<uChar,3> >("src1");
	const QVImage<uChar,3> src2 = getPropertyValue< QVImage<uChar,3> >("src2");
	const int scale = getPropertyValue< int >("scale");
	const QPoint destROIOffset = getPropertyValue< QPoint >("destROIOffset");
	timeFlag("Read input parameters");

	// Create output variables
	QVImage<uChar,3> dest(src1.getCols() + destROIOffset.x(), src1.getRows() + destROIOffset.y());

	Add(src1, src2, dest, scale, destROIOffset);
	timeFlag("Call to ippiAdd_8u_C3R");

	setPropertyValue< QVImage<uChar,3> >("dest", dest);
	timeFlag("Write output parameters");
	}

// qvippAddCWorker<uChar, 1>
template <> qvippAddCWorker<uChar,1>::qvippAddCWorker(QString name): QVWorker(name)
	{
	addProperty< QVImage<uChar,1> >("src", inputFlag|outputFlag);
	addProperty< int >("summand for channel 1", inputFlag, 128, "", 0, 255);
	addProperty< QPoint >("destROIOffset", inputFlag, QPoint(0,0));

	addProperty< QVImage<uChar,1> >("dest", outputFlag);
	}

template <> void qvippAddCWorker<uChar,1>::iterate()
	{
	// Read input properties
	const QVImage<uChar,1> src = getPropertyValue< QVImage<uChar,1> >("src");
	const int channel1Summand = getPropertyValue< int >("summand for channel 1");
	const QPoint destROIOffset = getPropertyValue< QPoint >("destROIOffset");
	timeFlag("Read input parameters");

	// Create output variables
	CREATE_OUTPUT_IMAGE(uChar, 1, dest, src, destROIOffset);

	AddC(src, channel1Summand, dest, 1, destROIOffset);
	timeFlag("Call to ippiAddC_8u_C1R");

	setPropertyValue< QVImage<uChar,1> >("dest", dest);
	timeFlag("Write output parameters");
	}

// qvippAddCWorker<uChar, 3>
template <> qvippAddCWorker<uChar,3>::qvippAddCWorker(QString name): QVWorker(name)
	{
	addProperty< QVImage<uChar,3> >("src", inputFlag|outputFlag);
	addProperty< int >("summand for channel 1", inputFlag, 128, "", 0, 255);
	addProperty< int >("summand for channel 2", inputFlag, 128, "", 0, 255);
	addProperty< int >("summand for channel 3", inputFlag, 128, "", 0, 255);
	addProperty< QPoint >("destROIOffset", inputFlag, QPoint(0,0));

	addProperty< QVImage<uChar,3> >("dest", outputFlag);
	}

template <> void qvippAddCWorker<uChar,3>::iterate()
	{
	// Read input properties
	const QVImage<uChar,3> src = getPropertyValue< QVImage<uChar,3> >("src");
	const int channel1Summand = getPropertyValue< int >("summand for channel 1");
	const int channel2Summand = getPropertyValue< int >("summand for channel 2");
	const int channel3Summand = getPropertyValue< int >("summand for channel 3");
	const QPoint destROIOffset = getPropertyValue< QPoint >("destROIOffset");
	timeFlag("Read input parameters");

	// Create output variables
	QVImage<uChar,3> dest(src.getCols() + destROIOffset.x(), src.getRows() + destROIOffset.y());

	const uChar channelSummands[3] = {channel1Summand, channel2Summand, channel3Summand};
	AddC(src, channelSummands, dest, 1, destROIOffset);
	timeFlag("Call to ippiAddC_8u_C3R");

	setPropertyValue< QVImage<uChar,3> >("dest", dest);
	timeFlag("Write output parameters");
	}

// qvippCompareCWorker<uChar, 1>
template <> qvippCompareCWorker<uChar,1>::qvippCompareCWorker(QString name): QVWorker(name)
	{
	addProperty< QVImage<uChar,1> >("src", inputFlag|outputFlag);
	ADD_IPP_CMP_OP_PROPERTY(operation);
	addProperty< int >("operand for channel 1", inputFlag);
	addProperty< QPoint >("destROIOffset", inputFlag, QPoint(0,0));

	addProperty< QVImage<uChar,1> >("dest", outputFlag);
	}

template <> void qvippCompareCWorker<uChar,1>::iterate()
	{
	// Read input properties
	const QVImage<uChar,1> src = getPropertyValue< QVImage<uChar,1> >("src");
	GET_IPP_CMP_OP_PROPERTY(operation);
	const int channel1Operand = getPropertyValue< int >("operand for channel 1");
	const QPoint destROIOffset = getPropertyValue< QPoint >("destROIOffset");
	timeFlag("Read input parameters");

	// Create output variables
	CREATE_OUTPUT_IMAGE(uChar, 1, dest, src, destROIOffset);

	CompareC(src, channel1Operand, dest, operation, destROIOffset);
	timeFlag("Call to ippiCompareC_8u_C1R");

	setPropertyValue< QVImage<uChar,1> >("dest", dest);
	timeFlag("Write output parameters");
	}

const QStringList getQvippWorkersNames()
	{
	QStringList result;

	result.append("qvippCopyP3C3Worker<uChar>");
	result.append("qvippCopyC3P3Worker<uChar>");
	result.append("qvippErode3x3Worker<uChar,1>");
	result.append("qvippComputeThreshold_OtsuWorker<uChar,1>");
	result.append("qvippAddRandUniform_DirectWorker<uChar,1>");
	result.append("qvippFilterBoxWorker<uChar,1>");
	result.append("qvippFilterBoxWorker<uChar,3>");
	result.append("qvippFilterMedianWorker<uChar,1>");
	result.append("qvippFilterMedianWorker<uChar,3>");
	result.append("qvippFilterMaxWorker<uChar,1>");
	result.append("qvippFilterMaxWorker<uChar,1>");
	result.append("qvippFilterMinWorker<uChar,1>");
	result.append("qvippFilterMinWorker<uChar,3>");
	result.append("qvippFilterGaussWorker<uChar,1>");
	result.append("qvippFilterHipassWorker<uChar,1>");
	result.append("qvippFilterGaussWorker<uChar,3>");
	result.append("qvippFilterHipassWorker<uChar,3>");
	result.append("qvippAddWorker<uChar,1>");
	result.append("qvippAddCWorker<uChar, 1>");
	result.append("qvippAddCWorker<uChar, 3>");
	result.append("qvippCompareCWorker<uChar, 1>");

	return result;
	}

QVPropertyContainer *newQvippWorker(QString type, QString name)
	{
	if (type == "qvippCopyP3C3Worker<uChar>")			return new qvippCopyP3C3Worker<uChar>(name);
	else if (type == "qvippCopyC3P3Worker<uChar>")			return new qvippCopyC3P3Worker<uChar>(name);
	else if (type == "qvippComputeThreshold_OtsuWorker<uChar,1>")	return new qvippComputeThreshold_OtsuWorker<uChar,1>(name);
	else if (type == "qvippAddRandUniform_DirectWorker<uChar,1>")	return new qvippAddRandUniform_DirectWorker<uChar,1>(name);
	else if (type == "qvippFilterBoxWorker<uChar,1>")		return new qvippFilterBoxWorker<uChar,1>(name);
	else if (type == "qvippFilterBoxWorker<uChar,3>")		return new qvippFilterBoxWorker<uChar,3>(name);
	else if (type == "qvippFilterMedianWorker<uChar,1>")		return new qvippFilterMedianWorker<uChar,1>(name);
	else if (type == "qvippFilterMedianWorker<uChar,3>")		return new qvippFilterMedianWorker<uChar,3>(name);
	else if (type == "qvippFilterMaxWorker<uChar,1>")		return new qvippFilterMaxWorker<uChar,1>(name);
	else if (type == "qvippFilterMaxWorker<uChar,3>")		return new qvippFilterMaxWorker<uChar,3>(name);
	else if (type == "qvippFilterMinWorker<uChar,1>")		return new qvippFilterMinWorker<uChar,1>(name);
	else if (type == "qvippFilterMinWorker<uChar,3>")		return new qvippFilterMinWorker<uChar,3>(name);
	else if (type == "qvippFilterGaussWorker<uChar,1>")		return new qvippFilterGaussWorker<uChar,1>(name);
	else if (type == "qvippFilterGaussWorker<uChar,3>")		return new qvippFilterGaussWorker<uChar,3>(name);
	else if (type == "qvippFilterHipassWorker<uChar,1>")		return new qvippFilterHipassWorker<uChar,1>(name);
	else if (type == "qvippFilterHipassWorker<uChar,3>")		return new qvippFilterHipassWorker<uChar,3>(name);
	else if (type == "qvippAddWorker<uChar,1>")	 		return new qvippAddWorker<uChar,1>(name);
	else if (type == "qvippAddCWorker<uChar, 1>")	 		return new qvippAddCWorker<uChar, 1>(name);
	else if (type == "qvippAddCWorker<uChar, 3>")	 		return new qvippAddCWorker<uChar, 3>(name);
	else if (type == "qvippCompareCWorker<uChar, 1>")		return new qvippCompareCWorker<uChar, 1>(name);
	else return NULL;
	}

